@@ -1,9 +1,12 @@
-import os
 import re
 
 from concurrent.futures import as_completed
+from os import name as os_name
+from pathlib import Path
 from requests_futures.sessions import FuturesSession
 
+from pathvalidate import sanitize_filepath
+
 from pytr.utils import preview, Timeline, get_logger
 
 
@@ -16,7 +19,7 @@ class DL:
         since_timestamp: downloaded files since this date (unix timestamp)
         '''
         self.tr = tr
-        self.output_path = output_path
+        self.output_path = Path(output_path)
         self.filename_fmt = filename_fmt
         self.since_timestamp = since_timestamp
 
@@ -64,7 +67,7 @@ class DL:
             time = f' {time[0]}'
 
         if subfolder is not None:
-            directory = os.path.join(self.output_path, subfolder)
+            directory = self.output_path / subfolder
         else:
             directory = self.output_path
 
@@ -82,15 +85,17 @@ class DL:
         filename = self.filename_fmt.format(
             iso_date=iso_date, time=time, title=titleText, subtitle=subtitleText, doc_num=doc_type_num
         )
-        if os.name == 'nt':
+        if os_name == 'nt':
             badChars = ['/', '\n', ':', '@', '.']
             for badChar in badChars:
                 filename = filename.replace(badChar, '')
 
         if doc_type in ['Kontoauszug', 'Depotauszug']:
-            filepath = os.path.join(directory, 'Abschlüsse', f'{filename}', f'{doc_type}.pdf')
+            filepath = directory / 'Abschlüsse' / f'{filename}', f'{doc_type}.pdf'
         else:
-            filepath = os.path.join(directory, doc_type, f'{filename}.pdf')
+            filepath = directory / doc_type / f'{filename}.pdf'
+
+        filepath = sanitize_filepath(filepath, '_', os_name)
 
         if filepath in self.filepaths:
             self.log.debug(f'File {filepath} already in queue. Skipping...')
@@ -98,7 +103,7 @@ class DL:
         else:
             self.filepaths.append(filepath)
 
-        if os.path.isfile(filepath) is False:
+        if filepath.is_file() is False:
             doc_url_base = doc_url.split('?')[0]
             if doc_url_base in self.doc_urls:
                 self.log.debug(f'URL {doc_url_base} already in queue. Skipping...')
@@ -122,16 +127,16 @@ class DL:
 
         self.log.info('Waiting for downloads to complete..')
         for future in as_completed(self.futures):
-            if os.path.isfile(future.filepath) is True:
+            if future.filepath.is_file() is True:
                 self.log.debug(f'file {future.filepath} was already downloaded.')
 
             r = future.result()
-            os.makedirs(os.path.dirname(future.filepath), exist_ok=True)
+            future.filepath.parent.mkdir(parents=True, exist_ok=True)
             with open(future.filepath, 'wb') as f:
                 f.write(r.content)
                 self.done += 1
 
-                self.log.debug(f'{self.done:>3}/{len(self.doc_urls)} {os.path.basename(future.filepath)}')
+                self.log.debug(f'{self.done:>3}/{len(self.doc_urls)} {future.filepath.name}')
 
                 if self.done == len(self.doc_urls):
                     self.log.info('Done.')
